import os
import datetime
from docutils import nodes, utils
from docutils.writers import html4css1
from atom import data
try:
    from rst2atom import sourcecode
except ImportError:
    pass
from rst2atom import rfc3339
try:
    from docutils.frontend import filter_settings_spec
except ImportError:
    def filter_settings_spec(settings_spec, *exclude, **replace):
        """Return a copy of `settings_spec` excluding/replacing some settings.

        `settings_spec` is a tuple of configuration settings with a structure
        described for docutils.SettingsSpec.settings_spec.

        Optional positional arguments are names of to-be-excluded settings.
        Keyword arguments are option specification replacements.
        (See the html4strict writer for an example.)
        """
        settings = list(settings_spec)
        # every third item is a sequence of option tuples
        for i in range(2, len(settings), 3):
            newopts = []
            for opt_spec in settings[i]:
                # opt_spec is ("<help>", [<option strings>], {<keyword args>})
                opt_name = [opt_string[2:].replace('-', '_')
                            for opt_string in opt_spec[1]
                                if opt_string.startswith('--')
                           ][0]
                if opt_name in exclude:
                    continue
                if opt_name in replace.keys():
                    newopts.append(replace[opt_name])
                else:
                    newopts.append(opt_spec)
            settings[i] = tuple(newopts)
        return tuple(settings)


__version__ = '0.1'
__author__ = 'Marco Giusti'


class Writer(html4css1.Writer):

    default_template_path = utils.relative_path(
        os.path.join(os.getcwd(), 'dummy'),
        os.path.join(os.path.dirname(__file__), 'template.txt'))

    settings_spec = filter_settings_spec(
        html4css1.Writer.settings_spec,
        'no_xml_declaration',
        'link_stylesheet',
        'stylesheet',
        'stylesheet_path',
        'embed_stylesheet')

    settings_defaults = {
        'xml_declaration': False,
        'embed_stylesheet': False,
        'stylesheet_path': False,
        'stylesheet': (),
        'template': default_template_path
    }

    visitor_attributes = ('header', 'body_pre_docinfo', 'docinfo', 'body',
                          'title', 'subtitle', 'footer', 'fragment')

    def __init__(self):
        html4css1.Writer.__init__(self)
        self.translator_class = AtomTranslator

    def apply_template(self):
        content = html4css1.Writer.apply_template(self)
        self.visitor.entry.content = data.Content(content)
        return str(self.visitor.feed)


category_delimiters = ('; ', ', ')


class AtomTranslator(html4css1.HTMLTranslator):

    def __init__(self, document):
        html4css1.HTMLTranslator.__init__(self, document)
        self.feed = data.Feed()
        self.entry = data.Entry()
        self.feed.entry.append(self.entry)
        self.feed.generator = data.Generator('rst2atom '+__version__)
        self.in_category = 0

        # if subtitle:
        #     feed.subtitle = data.Subtitle(text=subtitle)
        # if self.visitor.summary:
        #     entry.summary = data.Summary(text=self.visitor.summary)

    def visit_date(self, node):
        date = datetime.datetime.strptime(node.astext(), '%Y-%m-%d')
        date = rfc3339.rfc3339(date)
        self.entry.published = data.Published(date)

    def visit_author(self, node):
        if isinstance(node.parent, nodes.authors):
            contributor = data.Contributor(name=data.Name(node.astext()))
            self.entry.contributor.append(contributor)
        else:
            author = data.Author(name=data.Name(node.astext()))
            self.entry.author.append(author)

    def visit_contact(self, node):
        if isinstance(node.parent, nodes.author) and len(self.entry.author):
            self.entry.author[-1].email = data.Email(node.astext())
        elif len(self.entry.contributor):
            self.entry.contributor[-1].email = data.Email(node.astext())

    def visit_copyright(self, node):
        self.entry.rights = data.Rights(node.astext())

    def visit_title(self, node):
        if isinstance(node.parent, nodes.document):
            self.feed.title = self.entry.title = data.Title(node.astext())
        html4css1.HTMLTranslator.visit_title(self, node)

    def visit_field_name(self, node):
        name = node.astext().strip()
        if name == 'Category':
            self.in_category = 1
        html4css1.HTMLTranslator.visit_field_name(self, node)

    def visit_field_body(self, node):
        if self.in_category:
            body = node.astext().strip()
            categories = []
            for delimiter in category_delimiters:
                if delimiter in body:
                    categories = body.split(delimiter)
                    break
            for category in categories:
                self.entry.category.append(data.Category(term=category))
        html4css1.HTMLTranslator.visit_field_body(self, node)

    def depart_field_body(self, node):
        if self.in_category:
            self.in_category = 0
        html4css1.HTMLTranslator.depart_field_body(self, node)


    depart_author = depart_date = depart_copyright = depart_contact = \
            lambda s, n: None


def main():
    try:
        import locale
        locale.setlocale(locale.LC_ALL, '')
    except:
        pass
    from docutils.core import publish_cmdline, default_description

    description = ('Generates Atoms documents from standalone'
                   'reStructuredText sources.  ' + default_description)

    publish_cmdline(writer_name='rst2atom', description=description)

